import json
import logging
import six
from ipalib import api, errors, _
from ipalib import Command, Str, Int
from ipalib.plugable import Registry
from . import baseuser
from .baseuser import (baseuser, baseuser_output_params, validate_nsaccountlock)
from .baseldap import (LDAPObject, LDAPCreate, LDAPDelete,
                                     LDAPUpdate, LDAPSearch, LDAPRetrieve,
                                     LDAPAddMember, LDAPRemoveMember)
from ipalib import ngettext
from ipapython.dn import DN

if six.PY3:
    unicode = str


__doc__ = _("""
Parsec capabilities

Manage Parsec capabilities.

""")

register = Registry()

user_output_params = baseuser_output_params

#topic = 'pcaps'

def change_caps_attrs(ldap, dn, action, *keys, **options):
    """Формирование атрибута x-ald-user-cap каждый раз из значений атрибута memberof_pcaps 
    без учета предыдущего значения x-ald-user-cap (BT-62957)
    """
    if options.get('user'):
        for user in options.get('user'):
            user_attrs = api.Command.user_show(user)['result']
            (caplists, truncated) = ldap.find_entries(
                None, ['*'],
                DN(('cn', 'parsec-caps'), ('cn', 'caps'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
                ldap.SCOPE_ONELEVEL
            )

            caps_map = {} # Словарь соответствия имен_привилегии и их масок
            for caplist in caplists:
                caps_map[(caplist['cn'][0])] = (caplist['x-ald-user-cap'][0])

            sumcaps = 0 # Формирование действующей суммарной маски пользователя как суммы отдельных масок
            pcaps = user_attrs.get('memberof_pcaps', []) # названия имеющихся привилегий у пользователя
            for pcap in pcaps:
                mask = caps_map.get(pcap)
                sumcaps = sumcaps | int(mask, 16)
            for name in keys:
                if action == "del":
                    sumcaps = sumcaps & ~int(caps_map.get(name), 16) # новое значение суммарной маски пользователя
                else: # т.е. action == "add"
                    sumcaps = sumcaps | int(caps_map.get(name), 16)  # новое значение суммарной маски пользователя
            user_caps = user_attrs['x-ald-user-caps'][0] # старое значение суммарной маски пользователя 
            (parsec,linux) = user_caps.split(":")
            new_user_caps = hex(sumcaps) + ":" + linux
            #user_attrs.update({'x-ald-user-caps': new_user_caps}) # Вроде лишнее, но это не точно
            try:
                api.Command.user_mod(user, setattr=("x-ald-user-caps=%s" % new_user_caps,))
            except:
                logging.exception("ldap_update:")



def get_dn(self, *keys, **kwargs):

    try:
        if keys[-1] is not None:
           return self.api.Object.user.get_dn(*keys, **kwargs)
    except:
          logging.exception("get_dn:")
    return DN(self.container_dn, api.env.basedn)


@register()
class pcaps(LDAPObject):
    """
    CAP Level
    """
    container_dn = DN(('cn', 'parsec-caps'),
                      ('cn', 'caps'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('pcaps')
    object_name_plural = _('pcaps_s')
    object_class = ['x-ald-user', 'nestedgroup', 'groupofnames', 'nsContainer']
    default_attributes = ['cn', 'x-ald-user-cap', 'member']
    search_attributes = ['cn', 'x-ald-user-cap', 'member']

    attribute_members = {
        'member': ['user'],
    }

    label = _('CAP Levels')
    label_singular = _('CAP Level')

    takes_params = (
        Str('cn',
            cli_name='pcapname',
            label=_('Parsec CAP name'),
            primary_key=True,
        ),
        Int('x-ald-user-cap',
            cli_name='pcapid',
            label=_('Parsec CAP rank'),
            doc=_('integer to order the Parsec CAP rank'),
        ),
        Str('uid?',
            cli_name='user',
            label=_('User name'),
            doc=_('Manage CAPs for specific user'),
        ),
        Str('memberuser_user?',
            label=_('Users'),
        ),
    )

@register()
class pcaps_add(LDAPCreate):
    __doc__ = _('Create new Parsec CAP')

    msg_summary = _('Added Parsec CAP"%(value)s"')


@register()
class pcaps_del(LDAPDelete):
    __doc__ = _('Delete Parsec CAP.')

    msg_summary = _('Deleted Parsec CAP"%(value)s"')


@register()
class pcaps_mod(LDAPUpdate):
    __doc__ = _('Modify Parsec CAP.')

    msg_summary = _('Modified Parsec CAP"%(value)s"')


@register()
class pcaps_find(LDAPSearch):
    __doc__ = _('Search for Parsec CAP.')

    msg_summary = ngettext(
        '%(count)d Parsec CAP matched',
        '%(count)d Parsec CAP matched', 0
    )


@register()
class pcaps_show(LDAPRetrieve):
    __doc__ = _('Display Parsec CAP.')


@register()
class pcaps_add_pcaps(LDAPAddMember):

    __doc__ = _('Modify user CAPs.')

    member_attributes = ['member']
    has_output_params = LDAPAddMember.has_output_params

    logging.basicConfig(level=logging.DEBUG, filename='/tmp/myapp.log')

    def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
        assert isinstance(dn, DN)

        for memberof in list(options.get('user', [])):

           user = memberof

           user_attrs = {}
           user_attrs = api.Command.user_show(user)['result']

           if not 'x-ald-user-caps' in user_attrs:
             try:

                keys = [user]
                kw = {'setattr': unicode('x-ald-user-caps=%s' % '0x0:0x0')}
                api.Command['user_mod'](keys[-1], **kw)

             except:
               logging.exception("ldap_update:")

        return dn

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        change_caps_attrs(ldap, dn, "add", *keys, **options) # BT-62957

        return (completed, dn)


@register()
class pcaps_remove_member(LDAPRemoveMember):
    __doc__ = _('Remove users CAPs.')

    member_attributes = ['member']
    has_output_params = LDAPRemoveMember.has_output_params

    logging.basicConfig(level=logging.DEBUG, filename='/tmp/myapp.log')

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        change_caps_attrs(ldap, dn, "del", *keys, **options) # BT-62957

        return (completed, dn)
